#!/bin/bash

driverContainerName="v10sp1Driver"
ip_address=$(sudo docker inspect -f '{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}}' $driverContainerName)

get_usb_device() {
    local usb_devices=$(lpinfo -v | grep usb |grep $1)
    if [ -z "$usb_devices" ]; then
        echo "No USB devices found."
        return 1
    fi
    local usb_device=$(echo "$usb_devices" | head -n 1 | awk '{print $2}')
    echo "$usb_device"
}

setup_printer() {
    local printer_name=$1
    local printer_uri=$2
    local printer_display=$(echo $printer_name | sed 's/ /_/g')

    if [ -z "$printer_name" ]; then
        echo "No printer found."
        return 1
    fi

    sudo docker exec -t -u root $driverContainerName /bin/bash -c "/bin/bash /usr/bin/setup_printer.sh '$printer_name' '$printer_uri'"

    echo "waiting for printer to broadcast......."
    # 设置最大重试次数和间隔时间（秒）
    local max_retries=10
    local retry_interval=2
    local ipp_address=""

    for ((i=0; i<max_retries; i++)); do
        # 获取原始IPP地址并替换IP部分
        raw_address=$(ippfind | grep "$printer_display" | head -n 1)
        [ -n "$raw_address" ] && {
            # 替换IP地址部分（保留端口号）
            ipp_address=$(echo $raw_address | sed "s/ipp:\/\/[^:]*:/ipp:\/\/$ip_address:/g")
            break
        }
        sleep $retry_interval
    done

    if [ -z "$ipp_address" ]; then
        echo "Printer not found after $((max_retries * retry_interval)) seconds"
        return 1
    fi

    echo "execute lpadmin command: lpadmin -p $printer_display -v $ipp_address -E -m everywhere"
    sudo lpadmin -p "$printer_display" -v "$ipp_address" -E -m everywhere
}

get_usb_printer() {
    lsusb_ids=$(lsusb | awk '{print $6}')
    for id in $lsusb_ids; do
        is_printer=$(lsusb -v -d $id 2>>/dev/null| grep -i "7 Printer")
        if [ -z "$is_printer" ]; then
            continue
        fi
        device_product=$(lsusb -v -d $id | grep -i "iProduct" | awk '{print substr($0, index($0, $3))}') 
        device_serial=$(lsusb -v -d $id | grep -i "iSerial" | awk '{print substr($0, index($0, $3))}')
        echo "setup printer $device_product $device_serial"
        local device_uri=$(get_usb_device "$device_serial")
        if [ $? -ne 0 ]; then
            echo "No USB device found for $device_product."
            continue
        fi
        setup_printer "$device_product" "$device_uri"
    done
}

main() {
    if [ $# -eq 2 ]; then
        setup_printer "$1" "$2"
    elif [ $# -eq 0 ]; then
        get_usb_printer
    else
        echo "Usage: \$0 [printer_name printer_uri]"
        exit 1
    fi
}

main "$@"